@@ -1,4 +1,4 @@
-use crate::command::{self, CommandType, Invoke, PathArgs, WorkingDir};
+use crate::command::{self, Invoke, LintOrTidyCommandType, PathArgs, WorkingDir};
 use anyhow::Result;
 use indexmap::IndexMap;
 //use log::warn;
@@ -14,49 +14,52 @@ use thiserror::Error;
 #[derive(Clone, Debug, Deserialize)]
 pub struct CommandConfig {
     #[serde(rename = "type")]
-    typ: CommandType,
+    pub(crate) typ: LintOrTidyCommandType,
     #[serde(deserialize_with = "string_or_seq_string")]
-    include: Vec<String>,
+    pub(crate) include: Vec<String>,
     #[serde(default)]
     #[serde(deserialize_with = "string_or_seq_string")]
-    exclude: Vec<String>,
+    pub(crate) exclude: Vec<String>,
     #[serde(default)]
-    invoke: Option<Invoke>,
+    pub(crate) invoke: Option<Invoke>,
     #[serde(default)]
     #[serde(deserialize_with = "working_dir")]
-    working_dir: Option<WorkingDir>,
+    pub(crate) working_dir: Option<WorkingDir>,
     #[serde(default)]
-    path_args: Option<PathArgs>,
+    pub(crate) path_args: Option<PathArgs>,
     #[serde(default)]
-    run_mode: Option<OldRunMode>,
+    pub(crate) run_mode: Option<OldRunMode>,
     #[serde(default)]
-    chdir: Option<bool>,
+    pub(crate) chdir: Option<bool>,
     #[serde(deserialize_with = "string_or_seq_string")]
-    cmd: Vec<String>,
+    pub(crate) cmd: Vec<String>,
     #[serde(default)]
-    env: HashMap<String, String>,
+    pub(crate) env: HashMap<String, String>,
     #[serde(default)]
     #[serde(deserialize_with = "string_or_seq_string")]
-    lint_flags: Vec<String>,
+    pub(crate) lint_flags: Vec<String>,
     #[serde(default)]
     #[serde(deserialize_with = "string_or_seq_string")]
-    tidy_flags: Vec<String>,
+    pub(crate) tidy_flags: Vec<String>,
     #[serde(default = "empty_string")]
-    path_flag: String,
+    pub(crate) path_flag: String,
     #[serde(deserialize_with = "u8_or_seq_u8")]
-    ok_exit_codes: Vec<u8>,
+    pub(crate) ok_exit_codes: Vec<u8>,
     #[serde(default)]
     #[serde(deserialize_with = "u8_or_seq_u8")]
-    lint_failure_exit_codes: Vec<u8>,
+    pub(crate) lint_failure_exit_codes: Vec<u8>,
     #[serde(default)]
-    expect_stderr: bool,
+    pub(crate) expect_stderr: bool,
     #[serde(default)]
     #[serde(deserialize_with = "string_or_seq_string")]
-    ignore_stderr: Vec<String>,
+    pub(crate) ignore_stderr: Vec<String>,
+    #[serde(default)]
+    #[serde(deserialize_with = "string_or_seq_string")]
+    pub(crate) labels: Vec<String>,
 }
 
 #[derive(Clone, Copy, Debug, Deserialize, Eq, PartialEq)]
-pub enum OldRunMode {
+pub(crate) enum OldRunMode {
     #[serde(rename = "files")]
     Files,
     #[serde(rename = "dirs")]
@@ -73,12 +76,12 @@ fn empty_string() -> String {
 pub struct Config {
     #[serde(default)]
     #[serde(deserialize_with = "string_or_seq_string")]
-    pub exclude: Vec<String>,
+    pub(crate) exclude: Vec<String>,
     commands: IndexMap<String, CommandConfig>,
 }
 
 #[derive(Debug, Error, PartialEq, Eq)]
-pub enum ConfigError {
+pub(crate) enum ConfigError {
     #[error("File at {} cannot be read: {error:}", file.display())]
     FileCannotBeRead { file: PathBuf, error: String },
     #[error(
@@ -301,11 +304,11 @@ where
         where
             A: de::MapAccess<'de>,
         {
-            let mut kv_pairs: Vec<(&'de str, &'de str)> = vec![];
-            while let Some((k, v)) = map.next_entry::<&str, &str>()? {
-                if k != "chdir_to" {
+            let mut kv_pairs: Vec<(String, String)> = vec![];
+            while let Some((k, v)) = map.next_entry::<String, String>()? {
+                if &k != "chdir_to" {
                     return Err(<A::Error as de::Error>::invalid_value(
-                        de::Unexpected::Str(k),
+                        de::Unexpected::Str(&k),
                         &r#"the only valid key for a working_dir map is "chdir_to""#,
                     ));
                 }
@@ -332,55 +335,68 @@ where
                 ));
             }
 
-            Ok(Some(WorkingDir::ChdirTo(PathBuf::from(kv_pairs[0].1))))
+            Ok(Some(WorkingDir::ChdirTo(PathBuf::from(&kv_pairs[0].1))))
         }
     }
 
     deserializer.deserialize_any(WorkingDirOrChdirTo(PhantomData))
 }
 
+const DEFAULT_LABEL: &str = "default";
+
 impl Config {
-    pub fn new(file: PathBuf) -> Result<Config> {
-        match fs::read(&file) {
+    pub(crate) fn new(file: &Path) -> Result<Config> {
+        match fs::read(file) {
             Err(e) => Err(ConfigError::FileCannotBeRead {
-                file,
+                file: file.to_path_buf(),
                 error: e.to_string(),
             }
             .into()),
-            Ok(bytes) => Ok(toml::from_slice(&bytes)?),
+            Ok(bytes) => {
+                let s = String::from_utf8(bytes)?;
+                Ok(toml::from_str(&s)?)
+            }
         }
     }
 
-    pub fn into_tidy_commands(
+    pub(crate) fn into_tidy_commands(
         self,
         project_root: &Path,
         command: Option<&str>,
-    ) -> Result<Vec<command::Command>> {
-        self.into_commands(project_root, command, CommandType::Tidy)
+        label: Option<&str>,
+    ) -> Result<Vec<command::LintOrTidyCommand>> {
+        self.into_commands(project_root, command, label, LintOrTidyCommandType::Tidy)
     }
 
-    pub fn into_lint_commands(
+    pub(crate) fn into_lint_commands(
         self,
         project_root: &Path,
         command: Option<&str>,
-    ) -> Result<Vec<command::Command>> {
-        self.into_commands(project_root, command, CommandType::Lint)
+        label: Option<&str>,
+    ) -> Result<Vec<command::LintOrTidyCommand>> {
+        self.into_commands(project_root, command, label, LintOrTidyCommandType::Lint)
     }
 
     fn into_commands(
         self,
         project_root: &Path,
         command: Option<&str>,
-        typ: CommandType,
-    ) -> Result<Vec<command::Command>> {
-        let mut commands: Vec<command::Command> = vec![];
+        label: Option<&str>,
+        typ: LintOrTidyCommandType,
+    ) -> Result<Vec<command::LintOrTidyCommand>> {
+        let mut commands: Vec<command::LintOrTidyCommand> = vec![];
         for (name, c) in self.commands.into_iter() {
             if let Some(c) = command {
                 if name != c {
                     continue;
                 }
             }
-            if c.typ != typ && c.typ != CommandType::Both {
+
+            if !c.matches_label(label.unwrap_or(DEFAULT_LABEL)) {
+                continue;
+            }
+
+            if c.typ != typ && c.typ != LintOrTidyCommandType::Both {
                 continue;
             }
 
@@ -389,11 +405,15 @@ impl Config {
 
         Ok(commands)
     }
+
+    pub(crate) fn command_info(self) -> Vec<(String, CommandConfig)> {
+        self.commands.into_iter().collect()
+    }
 }
 
 impl CommandConfig {
-    fn into_command(self, project_root: &Path, name: String) -> Result<command::Command> {
-        let n = command::Command::new(self.into_command_params(project_root, name)?)?;
+    fn into_command(self, project_root: &Path, name: String) -> Result<command::LintOrTidyCommand> {
+        let n = command::LintOrTidyCommand::new(self.into_command_params(project_root, name)?)?;
         Ok(n)
     }
 
@@ -401,7 +421,7 @@ impl CommandConfig {
         self,
         project_root: &Path,
         name: String,
-    ) -> Result<command::CommandParams> {
+    ) -> Result<command::LintOrTidyCommandParams> {
         let (invoke, working_dir, path_args) = Self::invoke_args(
             &name,
             self.run_mode,
@@ -410,7 +430,7 @@ impl CommandConfig {
             self.working_dir,
             self.path_args,
         )?;
-        Ok(command::CommandParams {
+        Ok(command::LintOrTidyCommandParams {
             project_root: project_root.to_owned(),
             name,
             typ: self.typ,
@@ -505,6 +525,13 @@ impl CommandConfig {
 
         Ok((invoke, working_dir, path_args))
     }
+
+    fn matches_label(&self, label: &str) -> bool {
+        if self.labels.is_empty() {
+            return label == DEFAULT_LABEL;
+        }
+        self.labels.iter().any(|l| *l == label)
+    }
 }
 
 #[cfg(test)]
@@ -833,7 +860,7 @@ mod tests {
         expect_err: ConfigError,
     ) -> Result<()> {
         let config = CommandConfig {
-            typ: CommandType::Lint,
+            typ: LintOrTidyCommandType::Lint,
             invoke: Some(invoke),
             working_dir: Some(working_dir),
             path_args: Some(path_args),
@@ -850,6 +877,7 @@ mod tests {
             lint_failure_exit_codes: vec![],
             expect_stderr: false,
             ignore_stderr: vec![],
+            labels: vec![],
         };
         let res = config.into_command(Path::new("."), String::from("some-linter"));
         let err = res.unwrap_err().downcast::<ConfigError>().unwrap();
@@ -857,4 +885,47 @@ mod tests {
 
         Ok(())
     }
+
+    #[test_case(vec![], "default", true)]
+    #[test_case(vec!["default".to_string()], "default", true)]
+    #[test_case(vec!["default".to_string(), "foo".to_string()], "default", true)]
+    #[test_case(vec!["default".to_string(), "foo".to_string()], "foo", true)]
+    #[test_case(vec!["foo".to_string()], "foo", true)]
+    #[test_case(vec![], "foo", false)]
+    #[test_case(vec!["foo".to_string()], "default", false)]
+    #[test_case(vec!["default".to_string()], "foo", false)]
+    #[parallel]
+    fn matches_label(
+        labels_in_config: Vec<String>,
+        label_to_match: &str,
+        expect_match: bool,
+    ) -> Result<()> {
+        let config = CommandConfig {
+            typ: LintOrTidyCommandType::Lint,
+            invoke: None,
+            working_dir: None,
+            path_args: None,
+            include: vec![String::from("**/*.rs")],
+            exclude: vec![],
+            run_mode: None,
+            chdir: None,
+            cmd: vec![String::from("some-linter")],
+            env: Default::default(),
+            lint_flags: vec![],
+            tidy_flags: vec![],
+            path_flag: String::new(),
+            ok_exit_codes: vec![],
+            lint_failure_exit_codes: vec![],
+            expect_stderr: false,
+            ignore_stderr: vec![],
+            labels: labels_in_config,
+        };
+        if expect_match {
+            assert!(config.matches_label(label_to_match));
+        } else {
+            assert!(!config.matches_label(label_to_match));
+        }
+
+        Ok(())
+    }
 }
