@@ -1,21 +1,27 @@
 use crate::{
     chars,
     command::{self, TidyOutcome},
-    config,
+    config, config_init,
     paths::{self, finder::Finder},
     vcs,
 };
 use anyhow::{Error, Result};
-use clap::{AppSettings, ArgGroup, Parser};
+use clap::{ArgGroup, Parser, ValueEnum};
+use comfy_table::{presets::UTF8_FULL, Cell, ContentArrangement, Table};
 use fern::{
     colors::{Color, ColoredLevelConfig},
     Dispatch,
 };
+use indexmap::{IndexMap, IndexSet};
 use itertools::Itertools;
 use log::{debug, error, info};
 use rayon::{prelude::*, ThreadPool, ThreadPoolBuilder};
 use std::{
+    collections::{HashMap, HashSet},
     env,
+    fs::{create_dir_all, File},
+    io::stdout,
+    io::Write,
     path::{Path, PathBuf},
     time::{Duration, Instant},
 };
@@ -36,7 +42,13 @@ enum PreciousError {
     NoCommands { what: String },
 
     #[error("No {what:} commands match the given command name, {name:}")]
-    NoCommandsMatch { what: String, name: String },
+    NoCommandsMatchCommandName { what: String, name: String },
+
+    #[error("No {what:} commands match the given label, {label:}")]
+    NoCommandsMatchLabel { what: String, label: String },
+
+    #[error("There is already a precious.toml file in this directory")]
+    ConfigFileExists,
 }
 
 #[derive(Debug)]
@@ -63,14 +75,12 @@ struct ActionFailure {
     paths: Vec<PathBuf>,
 }
 
-const CONFIG_FILE_NAMES: &[&str] = &["precious.toml", ".precious.toml"];
-
 #[derive(Debug, Parser)]
 #[clap(name = "precious")]
 #[clap(author, version)]
 #[clap(propagate_version = true)]
-#[clap(global_setting(AppSettings::DeriveDisplayOrder))]
 #[clap(subcommand_required = true, arg_required_else_help = true)]
+#[clap(max_term_width = 100)]
 /// One code quality tool to rule them all
 pub struct App {
     /// Path to the precious config file
@@ -103,13 +113,14 @@ pub struct App {
 pub enum Subcommand {
     Lint(CommonArgs),
     Tidy(CommonArgs),
+    Config(ConfigArgs),
 }
 
 #[derive(Debug, Parser)]
 #[clap(group(
     ArgGroup::new("path-spec")
         .required(true)
-        .args(&["all", "git", "staged", "staged-with-stash", "paths"]),
+        .args(&["all", "git", "staged", "staged_with_stash", "paths"]),
 ))]
 pub struct CommonArgs {
     /// The command to run. If specified, only this command will be run. This
@@ -129,27 +140,48 @@ pub struct CommonArgs {
     /// unstaged content first
     #[clap(long)]
     staged_with_stash: bool,
+    /// If this is set, then only commands matching this label will be run. If
+    /// this isn't set then commands without a label or with the label
+    /// "default" will be run.
+    #[clap(long)]
+    label: Option<String>,
     /// A list of paths on which to operate
     #[clap(value_parser)]
     paths: Vec<PathBuf>,
 }
 
-pub fn app() -> App {
-    App::parse()
+#[derive(Debug, Parser)]
+pub struct ConfigArgs {
+    #[clap(subcommand)]
+    subcommand: ConfigSubcommand,
 }
 
-#[derive(Debug)]
-pub struct Precious {
-    mode: paths::mode::Mode,
-    project_root: PathBuf,
-    cwd: PathBuf,
-    config: config::Config,
-    command: Option<String>,
-    chars: chars::Chars,
-    quiet: bool,
-    thread_pool: ThreadPool,
-    should_lint: bool,
-    paths: Vec<PathBuf>,
+#[derive(Debug, Parser)]
+enum ConfigSubcommand {
+    List,
+    Init(ConfigInitArgs),
+}
+
+#[derive(Debug, Parser)]
+pub struct ConfigInitArgs {
+    #[clap(long, short, value_enum)]
+    component: Vec<InitComponent>,
+    #[clap(long, short, default_value = "./precious.toml")]
+    path: PathBuf,
+}
+
+#[derive(Clone, Copy, Debug, ValueEnum)]
+pub enum InitComponent {
+    Go,
+    Perl,
+    Rust,
+    Gitignore,
+    Markdown,
+    YAML,
+}
+
+pub fn app() -> App {
+    App::parse()
 }
 
 impl App {
@@ -191,10 +223,294 @@ impl App {
             .chain(std::io::stderr())
             .apply()
     }
+
+    pub fn run(self) -> Result<i8> {
+        self._run(stdout())
+    }
+
+    #[cfg(test)]
+    fn run_with_output(self, output: impl Write) -> Result<i8> {
+        self._run(output)
+    }
+
+    fn _run(self, output: impl Write) -> Result<i8> {
+        if let Subcommand::Config(config_args) = &self.subcommand {
+            if let ConfigSubcommand::Init(init_args) = &config_args.subcommand {
+                init_config(&init_args.component, &init_args.path)?;
+                return Ok(0);
+            }
+        }
+
+        let (cwd, project_root, config_file, config) = self.load_config()?;
+
+        match self.subcommand {
+            Subcommand::Lint(_) | Subcommand::Tidy(_) => {
+                Ok(LintOrTidyRunner::new(self, cwd, project_root, config)?.run())
+            }
+            Subcommand::Config(args) => {
+                match args.subcommand {
+                    ConfigSubcommand::List => {
+                        print_config(output, &config_file, config)?;
+                    }
+                    ConfigSubcommand::Init(_) => {
+                        unreachable!("This is handled earlier")
+                    }
+                }
+
+                Ok(0)
+            }
+        }
+    }
+
+    // This exists to make writing tests of the runner easier.
+    #[cfg(test)]
+    fn new_lint_or_tidy_runner(self) -> Result<LintOrTidyRunner> {
+        let (cwd, project_root, _, config) = self.load_config()?;
+        LintOrTidyRunner::new(self, cwd, project_root, config)
+    }
+
+    fn load_config(&self) -> Result<(PathBuf, PathBuf, PathBuf, config::Config)> {
+        let cwd = env::current_dir()?;
+        let project_root = project_root(self.config.as_ref(), &cwd)?;
+        let config_file = self.config_file(&project_root);
+        let config = config::Config::new(&config_file)?;
+
+        Ok((cwd, project_root, config_file, config))
+    }
+
+    fn config_file(&self, dir: &Path) -> PathBuf {
+        if let Some(cf) = self.config.as_ref() {
+            debug!("Loading config from {} (set via flag)", cf.display());
+            return cf.to_path_buf();
+        }
+
+        let default = default_config_file(dir);
+        debug!(
+            "Loading config from {} (default location)",
+            default.display()
+        );
+        default
+    }
 }
 
-impl Precious {
-    pub fn new(app: App) -> Result<Precious> {
+fn project_root(config_file: Option<&PathBuf>, cwd: &Path) -> Result<PathBuf> {
+    if let Some(file) = config_file {
+        if let Some(p) = file.parent() {
+            return Ok(p.to_path_buf());
+        }
+        return Err(PreciousError::ConfigFileHasNoParent {
+            file: file.to_path_buf(),
+        }
+        .into());
+    }
+
+    if has_config_file(cwd) {
+        return Ok(cwd.into());
+    }
+
+    for anc in cwd.ancestors() {
+        if is_checkout_root(anc) {
+            return Ok(anc.to_owned());
+        }
+    }
+
+    Err(PreciousError::CannotFindRoot {
+        cwd: cwd.to_string_lossy().to_string(),
+    }
+    .into())
+}
+
+fn has_config_file(dir: &Path) -> bool {
+    default_config_file(dir).exists()
+}
+
+const CONFIG_FILE_NAMES: &[&str] = &["precious.toml", ".precious.toml"];
+
+fn default_config_file(dir: &Path) -> PathBuf {
+    // It'd be nicer to use the version of this provided by itertools, but
+    // that requires itertools 0.10.1, and we want to keep the version at
+    // 0.9.0 for the benefit of Debian.
+    find_or_first(
+        CONFIG_FILE_NAMES.iter().map(|n| {
+            let mut path = dir.to_path_buf();
+            path.push(n);
+            path
+        }),
+        |p| p.exists(),
+    )
+}
+
+fn find_or_first<I, P>(mut iter: I, pred: P) -> PathBuf
+where
+    I: Iterator<Item = PathBuf>,
+    P: Fn(&Path) -> bool,
+{
+    let first = iter.next().unwrap();
+    if pred(&first) {
+        return first;
+    }
+    iter.find(|i| pred(i)).unwrap_or(first)
+}
+
+fn is_checkout_root(dir: &Path) -> bool {
+    for subdir in vcs::DIRS {
+        let mut poss = PathBuf::from(dir);
+        poss.push(subdir);
+        if poss.exists() {
+            return true;
+        }
+    }
+    false
+}
+
+fn print_config(mut output: impl Write, config_file: &Path, config: config::Config) -> Result<()> {
+    writeln!(output, "Found config file at: {}", config_file.display())?;
+    writeln!(output)?;
+
+    let mut table = Table::new();
+    table
+        .load_preset(UTF8_FULL)
+        .set_content_arrangement(ContentArrangement::Dynamic)
+        .set_header(vec![
+            Cell::new("Name"),
+            Cell::new("Type"),
+            Cell::new("Runs"),
+        ]);
+
+    for (name, c) in config.command_info() {
+        table.add_row(vec![
+            Cell::new(name),
+            Cell::new(c.typ),
+            Cell::new(c.cmd.join(" ")),
+        ]);
+    }
+    writeln!(output, "{table}")?;
+
+    Ok(())
+}
+
+fn init_config(components: &[InitComponent], path: &Path) -> Result<()> {
+    if has_config_file(&env::current_dir()?) {
+        return Err(PreciousError::ConfigFileExists.into());
+    }
+
+    let mut excludes: HashSet<&'static str> = HashSet::new();
+    let mut commands = IndexMap::new();
+    let mut extra_files = HashMap::new();
+    let mut tool_urls: IndexSet<&'static str> = IndexSet::new();
+
+    for l in components {
+        let init = match l {
+            InitComponent::Go => config_init::go_init(),
+            InitComponent::Perl => config_init::perl_init(),
+            InitComponent::Rust => config_init::rust_init(),
+            InitComponent::Gitignore => config_init::gitignore_init(),
+            InitComponent::Markdown => config_init::markdown_init(),
+            InitComponent::YAML => config_init::yaml_init(),
+        };
+        excludes.extend(init.excludes);
+        for (name, c) in init.commands {
+            commands.insert(name, c);
+        }
+        for (name, c) in init.extra_files {
+            extra_files.insert(name, c);
+        }
+        tool_urls.extend(init.tool_urls);
+    }
+
+    let mut toml = String::new();
+    if !excludes.is_empty() {
+        toml.push_str(&format!(
+            "excludes = [\n{}\n]",
+            excludes
+                .iter()
+                .sorted()
+                .map(|e| format!(r#"    "{e}","#))
+                .collect::<Vec<_>>()
+                .join("\n")
+        ));
+    }
+
+    if !toml.is_empty() {
+        toml.push_str("\n\n");
+    }
+
+    let mut command_strs: Vec<String> = Vec::new();
+
+    for (name, c) in commands {
+        let name_str = if name.contains(' ') {
+            format!(r#""{name}""#)
+        } else {
+            name.to_string()
+        };
+        command_strs.push(format!("[commands.{name_str}]\n{}\n", c.trim()));
+    }
+
+    toml.push_str(&command_strs.join("\n"));
+
+    println!();
+    println!("Writing {}", path.display());
+
+    let mut precious_toml = File::create(path)?;
+    precious_toml.write_all(toml.as_bytes())?;
+
+    if !extra_files.is_empty() {
+        println!();
+        println!("Generating support files");
+        println!();
+
+        let paths = extra_files.keys().sorted().collect::<Vec<_>>();
+
+        for p in paths {
+            print!("{} ...", p.display());
+            if p.exists() {
+                println!(
+                    "  already exists, skipping - delete this file if you want to regenerate it"
+                );
+                continue;
+            }
+            println!(" generated");
+
+            if let Some(parent) = p.parent() {
+                create_dir_all(parent)?;
+            }
+            let mut file = File::create(p)?;
+            file.write_all(extra_files.get(p).unwrap().trim_start().as_bytes())?;
+        }
+    }
+
+    println!();
+    println!("The generated precious.toml requires the following tools to be installed:");
+    for u in tool_urls {
+        println!("  {u}");
+    }
+    println!();
+
+    Ok(())
+}
+
+#[derive(Debug)]
+pub struct LintOrTidyRunner {
+    mode: paths::mode::Mode,
+    project_root: PathBuf,
+    cwd: PathBuf,
+    config: config::Config,
+    command: Option<String>,
+    chars: chars::Chars,
+    quiet: bool,
+    thread_pool: ThreadPool,
+    should_lint: bool,
+    paths: Vec<PathBuf>,
+    label: Option<String>,
+}
+
+impl LintOrTidyRunner {
+    pub fn new(
+        app: App,
+        cwd: PathBuf,
+        project_root: PathBuf,
+        config: config::Config,
+    ) -> Result<LintOrTidyRunner> {
         if log::log_enabled!(log::Level::Debug) {
             if let Some(path) = env::var_os("PATH") {
                 debug!("PATH = {}", path.to_string_lossy());
@@ -208,18 +524,15 @@ impl Precious {
         };
 
         let mode = Self::mode(&app)?;
-        let cwd = env::current_dir()?;
-        let project_root = Self::project_root(app.config.as_ref(), &cwd)?;
-        let config_file = Self::config_file(app.config.as_ref(), &project_root);
-        let config = config::Config::new(config_file)?;
         let quiet = app.quiet;
         let jobs = app.jobs;
-        let (should_lint, paths, command) = match app.subcommand {
-            Subcommand::Lint(a) => (true, a.paths, a.command),
-            Subcommand::Tidy(a) => (false, a.paths, a.command),
+        let (should_lint, paths, command, label) = match app.subcommand {
+            Subcommand::Lint(a) => (true, a.paths, a.command, a.label),
+            Subcommand::Tidy(a) => (false, a.paths, a.command, a.label),
+            Subcommand::Config(_) => unreachable!("this is handled in App::run"),
         };
 
-        Ok(Precious {
+        Ok(LintOrTidyRunner {
             mode,
             project_root,
             cwd,
@@ -230,6 +543,7 @@ impl Precious {
             thread_pool: ThreadPoolBuilder::new().num_threads(jobs).build()?,
             should_lint,
             paths,
+            label,
         })
     }
 
@@ -237,6 +551,7 @@ impl Precious {
         let common = match &app.subcommand {
             Subcommand::Lint(c) => c,
             Subcommand::Tidy(c) => c,
+            Subcommand::Config(_) => unreachable!("this is handled in App::run"),
         };
         if common.all {
             return Ok(paths::mode::Mode::All);
@@ -253,86 +568,7 @@ impl Precious {
         Ok(paths::mode::Mode::FromCli)
     }
 
-    fn project_root(file: Option<&PathBuf>, cwd: &Path) -> Result<PathBuf> {
-        if let Some(file) = file {
-            if let Some(p) = file.parent() {
-                return Ok(p.to_path_buf());
-            }
-            return Err(PreciousError::ConfigFileHasNoParent { file: file.clone() }.into());
-        }
-
-        if Self::has_config_file(cwd) {
-            return Ok(cwd.into());
-        }
-
-        for anc in cwd.ancestors() {
-            if Self::is_checkout_root(anc) {
-                return Ok(anc.to_owned());
-            }
-        }
-
-        Err(PreciousError::CannotFindRoot {
-            cwd: cwd.to_string_lossy().to_string(),
-        }
-        .into())
-    }
-
-    fn has_config_file(dir: &Path) -> bool {
-        Self::default_config_file(dir).exists()
-    }
-
-    fn default_config_file(dir: &Path) -> PathBuf {
-        // It'd be nicer to use the version of this provided by itertools, but
-        // that requires itertools 0.10.1, and we want to keep the version at
-        // 0.9.0 for the benefit of Debian.
-        Self::find_or_first(
-            CONFIG_FILE_NAMES.iter().map(|n| {
-                let mut path = dir.to_path_buf();
-                path.push(n);
-                path
-            }),
-            |p| p.exists(),
-        )
-    }
-
-    fn find_or_first<I, P>(mut iter: I, pred: P) -> PathBuf
-    where
-        I: Iterator<Item = PathBuf>,
-        P: Fn(&Path) -> bool,
-    {
-        let first = iter.next().unwrap();
-        if pred(&first) {
-            return first;
-        }
-        iter.find(|i| pred(i)).unwrap_or(first)
-    }
-
-    fn config_file(file: Option<&PathBuf>, dir: &Path) -> PathBuf {
-        if let Some(cf) = file {
-            debug!("Loading config from {} (set via flag)", cf.display());
-            return cf.to_path_buf();
-        }
-
-        let default = Self::default_config_file(dir);
-        debug!(
-            "Loading config from {} (default location)",
-            default.display()
-        );
-        default
-    }
-
-    fn is_checkout_root(dir: &Path) -> bool {
-        for subdir in vcs::DIRS {
-            let mut poss = PathBuf::from(dir);
-            poss.push(subdir);
-            if poss.exists() {
-                return true;
-            }
-        }
-        false
-    }
-
-    pub fn run(&mut self) -> i8 {
+    fn run(&mut self) -> i8 {
         match self.run_subcommand() {
             Ok(e) => {
                 debug!("{:?}", e);
@@ -367,11 +603,15 @@ impl Precious {
             // XXX - This clone can be removed if config is passed into this
             // method instead of being a field of self.
             .clone()
-            .into_tidy_commands(&self.project_root, self.command.as_deref())?;
+            .into_tidy_commands(
+                &self.project_root,
+                self.command.as_deref(),
+                self.label.as_deref(),
+            )?;
         self.run_all_commands(
             "tidying",
             tidiers,
-            |self_: &mut Self, files: &[PathBuf], tidier: &command::Command| {
+            |self_: &mut Self, files: &[PathBuf], tidier: &command::LintOrTidyCommand| {
                 self_.run_one_tidier(files, tidier)
             },
         )
@@ -384,11 +624,15 @@ impl Precious {
             .config
             // XXX - same as above.
             .clone()
-            .into_lint_commands(&self.project_root, self.command.as_deref())?;
+            .into_lint_commands(
+                &self.project_root,
+                self.command.as_deref(),
+                self.label.as_deref(),
+            )?;
         self.run_all_commands(
             "linting",
             linters,
-            |self_: &mut Self, files: &[PathBuf], linter: &command::Command| {
+            |self_: &mut Self, files: &[PathBuf], linter: &command::LintOrTidyCommand| {
                 self_.run_one_linter(files, linter)
             },
         )
@@ -397,20 +641,31 @@ impl Precious {
     fn run_all_commands<R>(
         &mut self,
         action: &str,
-        commands: Vec<command::Command>,
+        commands: Vec<command::LintOrTidyCommand>,
         run_command: R,
     ) -> Result<Exit>
     where
-        R: Fn(&mut Self, &[PathBuf], &command::Command) -> Result<Option<Vec<ActionFailure>>>,
+        R: Fn(
+            &mut Self,
+            &[PathBuf],
+            &command::LintOrTidyCommand,
+        ) -> Result<Option<Vec<ActionFailure>>>,
     {
         if commands.is_empty() {
             if let Some(c) = &self.command {
-                return Err(PreciousError::NoCommandsMatch {
+                return Err(PreciousError::NoCommandsMatchCommandName {
                     what: action.into(),
                     name: c.into(),
                 }
                 .into());
             }
+            if let Some(l) = &self.label {
+                return Err(PreciousError::NoCommandsMatchLabel {
+                    what: action.into(),
+                    label: l.into(),
+                }
+                .into());
+            }
             return Err(PreciousError::NoCommands {
                 what: action.into(),
             }
@@ -485,7 +740,7 @@ impl Precious {
     fn run_one_tidier(
         &mut self,
         files: &[PathBuf],
-        t: &command::Command,
+        t: &command::LintOrTidyCommand,
     ) -> Result<Option<Vec<ActionFailure>>> {
         let runner = |s: &Self, files: &[&Path]| -> Option<Result<(), ActionFailure>> {
             match t.tidy(files) {
@@ -545,7 +800,7 @@ impl Precious {
     fn run_one_linter(
         &mut self,
         files: &[PathBuf],
-        l: &command::Command,
+        l: &command::LintOrTidyCommand,
     ) -> Result<Option<Vec<ActionFailure>>> {
         let runner = |s: &Self, files: &[&Path]| -> Option<Result<(), ActionFailure>> {
             match l.lint(files) {
@@ -618,7 +873,7 @@ impl Precious {
         &mut self,
         what: &str,
         files: &[PathBuf],
-        c: &command::Command,
+        c: &command::LintOrTidyCommand,
         runner: R,
     ) -> Result<Option<Vec<ActionFailure>>>
     where
@@ -740,14 +995,9 @@ lint_failure_exit_codes = [1]
             let _pushd = helper.pushd_to_git_root()?;
 
             let app = App::try_parse_from(["precious", "tidy", "--all"])?;
-            let config = app.config.clone();
 
-            let p = Precious::new(app)?;
-            assert_eq!(p.chars, chars::FUN_CHARS);
-            assert!(!p.quiet);
-
-            let config_file = Precious::config_file(config.as_ref(), &p.project_root);
-            let mut expect_config_file = p.project_root;
+            let (_, project_root, config_file, _) = app.load_config()?;
+            let mut expect_config_file = project_root;
             expect_config_file.push(name);
             assert_eq!(config_file, expect_config_file);
         }
@@ -764,8 +1014,8 @@ lint_failure_exit_codes = [1]
 
         let app = App::try_parse_from(["precious", "--ascii", "tidy", "--all"])?;
 
-        let p = Precious::new(app)?;
-        assert_eq!(p.chars, chars::BORING_CHARS);
+        let lt = app.new_lint_or_tidy_runner()?;
+        assert_eq!(lt.chars, chars::BORING_CHARS);
 
         Ok(())
     }
@@ -787,11 +1037,9 @@ lint_failure_exit_codes = [1]
             "tidy",
             "--all",
         ])?;
-        let config = app.config.clone();
-        let p = Precious::new(app)?;
 
-        let config_file = Precious::config_file(config.as_ref(), &p.project_root);
-        let mut expect_config_file = p.project_root;
+        let (_, project_root, config_file, _) = app.load_config()?;
+        let mut expect_config_file = project_root;
         expect_config_file.push(DEFAULT_CONFIG_FILE_NAME);
         assert_eq!(config_file, expect_config_file);
 
@@ -812,8 +1060,8 @@ lint_failure_exit_codes = [1]
 
         let app = App::try_parse_from(["precious", "--quiet", "tidy", "--all"])?;
 
-        let p = Precious::new(app)?;
-        assert_eq!(p.project_root, src_dir);
+        let lt = app.new_lint_or_tidy_runner()?;
+        assert_eq!(lt.project_root, src_dir);
 
         Ok(())
     }
@@ -905,10 +1153,10 @@ lint_failure_exit_codes = [1]
         }
         let app = App::try_parse_from(&cmd)?;
 
-        let mut p = Precious::new(app)?;
+        let mut lt = app.new_lint_or_tidy_runner()?;
 
         assert_eq!(
-            p.finder()?
+            lt.finder()?
                 .files(paths.iter().map(PathBuf::from).collect())?,
             Some(expect.iter().map(PathBuf::from).collect::<Vec<_>>()),
             "finder_uses_project_root: {} [{}]",
@@ -935,8 +1183,8 @@ lint_failure_exit_codes = [1]
 
         let app = App::try_parse_from(["precious", "--quiet", "tidy", "--all"])?;
 
-        let mut p = Precious::new(app)?;
-        let status = p.run();
+        let mut lt = app.new_lint_or_tidy_runner()?;
+        let status = lt.run();
 
         assert_eq!(status, 0);
 
@@ -959,8 +1207,8 @@ lint_failure_exit_codes = [1]
 
         let app = App::try_parse_from(["precious", "--quiet", "tidy", "--all"])?;
 
-        let mut p = Precious::new(app)?;
-        let status = p.run();
+        let mut lt = app.new_lint_or_tidy_runner()?;
+        let status = lt.run();
 
         assert_eq!(status, 1);
 
@@ -984,8 +1232,8 @@ lint_failure_exit_codes = [1]
 
         let app = App::try_parse_from(["precious", "--quiet", "lint", "--all"])?;
 
-        let mut p = Precious::new(app)?;
-        let status = p.run();
+        let mut lt = app.new_lint_or_tidy_runner()?;
+        let status = lt.run();
 
         assert_eq!(status, 0);
 
@@ -1008,8 +1256,8 @@ lint_failure_exit_codes = [1]
             "--all",
         ])?;
 
-        let mut p = Precious::new(app)?;
-        let status = p.run();
+        let mut lt = app.new_lint_or_tidy_runner()?;
+        let status = lt.run();
 
         assert_eq!(status, 0);
 
@@ -1032,8 +1280,8 @@ lint_failure_exit_codes = [1]
             "--all",
         ])?;
 
-        let mut p = Precious::new(app)?;
-        let status = p.run();
+        let mut lt = app.new_lint_or_tidy_runner()?;
+        let status = lt.run();
 
         assert_eq!(status, 1);
 
@@ -1081,8 +1329,7 @@ lint_failure_exit_codes = [1]
 
         let app = App::try_parse_from(["precious", "--quiet", "tidy", "-a"])?;
 
-        let mut p = Precious::new(app)?;
-        let status = p.run();
+        let status = app.run()?;
 
         assert_eq!(status, 0);
 
@@ -1092,6 +1339,58 @@ lint_failure_exit_codes = [1]
         Ok(())
     }
 
+    #[test]
+    fn print_config() -> Result<()> {
+        let config = r#"
+            [commands.foo]
+            type    = "lint"
+            include = "*.foo"
+            cmd     = ["foo", "--lint", "--with-vigor"]
+            ok_exit_codes = [0]
+
+            [commands.bar]
+            type    = "tidy"
+            include = "*.bar"
+            cmd     = ["bar", "--fix-broken-things", "--aggressive"]
+            ok_exit_codes = [0]
+
+            [commands.baz]
+            type    = "both"
+            include = "*.baz"
+            cmd     = ["baz", "--fast-mode", "--no-verify"]
+            lint_flags = "--lint"
+            ok_exit_codes = [0]
+        "#;
+        let helper = TestHelper::new()?.with_config_file(DEFAULT_CONFIG_FILE_NAME, config)?;
+        let _pushd = helper.pushd_to_git_root()?;
+
+        let app = App::try_parse_from(["precious", "config", "list"])?;
+        let mut buffer = Vec::new();
+        let status = app.run_with_output(&mut buffer)?;
+
+        assert_eq!(status, 0);
+
+        let output = String::from_utf8(buffer)?;
+        let expect = format!(
+            r#"Found config file at: {}
+
+┌──────┬──────┬──────────────────────────────────────┐
+│ Name ┆ Type ┆ Runs                                 │
+╞══════╪══════╪══════════════════════════════════════╡
+│ foo  ┆ lint ┆ foo --lint --with-vigor              │
+├╌╌╌╌╌╌┼╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
+│ bar  ┆ tidy ┆ bar --fix-broken-things --aggressive │
+├╌╌╌╌╌╌┼╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
+│ baz  ┆ both ┆ baz --fast-mode --no-verify          │
+└──────┴──────┴──────────────────────────────────────┘
+"#,
+            helper.config_file(DEFAULT_CONFIG_FILE_NAME).display(),
+        );
+        assert_eq!(output, expect);
+
+        Ok(())
+    }
+
     #[test]
     fn format_duration_output() {
         let mut tests: HashMap<Duration, &'static str> = HashMap::new();
