@@ -9,9 +9,8 @@ fn main() {
         eprintln!("Error creating logger: {e}");
         std::process::exit(1);
     }
-    let p = precious::Precious::new(app);
-    let status = match p {
-        Ok(mut p) => p.run(),
+    let status = match app.run() {
+        Ok(s) => s,
         Err(e) => {
             error!("{e}");
             1
